@@ -1,5 +1,5 @@
 /*
- *  extern.h  -- header file for apcupsd package
+ *  apc_extern.h  -- header file for apcupsd package
  *
  *  apcupsd.c -- Simple Daemon to catch power failure signals from a
  *               BackUPS, BackUPS Pro, or SmartUPS (from APCC).
@@ -10,49 +10,23 @@
  */
 
 /*
- *                     GNU GENERAL PUBLIC LICENSE
- *                        Version 2, June 1991
- *
- *  Copyright (C) 1989, 1991 Free Software Foundation, Inc.
- *                           675 Mass Ave, Cambridge, MA 02139, USA
- *  Everyone is permitted to copy and distribute verbatim copies
- *  of this license document, but changing it is not allowed.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
- *
- */
+   Copyright (C) 2000-2004 Kern Sibbald
+
+   This program is free software; you can redistribute it and/or
+   modify it under the terms of the GNU General Public License as
+   published by the Free Software Foundation; either version 2 of
+   the License, or (at your option) any later version.
+
+   This program is distributed in the hope that it will be useful,
+   but WITHOUT ANY WARRANTY; without even the implied warranty of
+   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+   General Public License for more details.
+
+   You should have received a copy of the GNU General Public
+   License along with this program; if not, write to the Free
+   Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
+   MA 02111-1307, USA.
 
-/*
- *  IN NO EVENT SHALL ANY AND ALL PERSONS INVOLVED IN THE DEVELOPMENT OF THIS
- *  PACKAGE, NOW REFERRED TO AS "APCUPSD-Team" BE LIABLE TO ANY PARTY FOR
- *  DIRECT, INDIRECT, SPECIAL, INCIDENTAL, OR CONSEQUENTIAL DAMAGES ARISING
- *  OUT OF THE USE OF THIS SOFTWARE AND ITS DOCUMENTATION, EVEN IF ANY OR ALL
- *  OF THE "APCUPSD-Team" HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
- *
- *  THE "APCUPSD-Team" SPECIFICALLY DISCLAIMS ANY WARRANTIES, INCLUDING,
- *  BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
- *  FITNESS FOR A PARTICULAR PURPOSE.  THE SOFTWARE PROVIDED HEREUNDER IS
- *  ON AN "AS IS" BASIS, AND THE "APCUPSD-Team" HAS NO OBLIGATION TO PROVIDE
- *  MAINTENANCE, SUPPORT, UPDATES, ENHANCEMENTS, OR MODIFICATIONS.
- *
- *  THE "APCUPSD-Team" HAS ABSOLUTELY NO CONNECTION WITH THE COMPANY
- *  AMERICAN POWER CONVERSION, "APCC".  THE "APCUPSD-Team" DID NOT AND
- *  HAS NOT SIGNED ANY NON-DISCLOSURE AGREEMENTS WITH "APCC".  ANY AND ALL
- *  OF THE LOOK-A-LIKE ( UPSlink(tm) Language ) WAS DERIVED FROM THE
- *  SOURCES LISTED BELOW.
- *
  */
 
 #ifndef _APC_EXTERN_H
@@ -64,7 +38,15 @@
 
 extern UPSINFO myUPS;
 extern UPSINFO *core_ups;
+extern UPSDRIVER drivers[];
 extern char argvalue[MAXSTRING];
+extern void (*error_out)(char *file, int line, char *fmt,...);
+extern void (*error_exit)(char *fmt,...);
+extern void (*error_cleanup)(void);
+
+extern UPSCOMMANDS ups_event[];
+extern UPSCMDMSG event_msg[];
+
 
 /*
  * Serial bits
@@ -84,13 +66,12 @@ extern int dsr_bit;
  */
 extern int flags;
 
-extern struct termios oldtio;
 extern struct termios newtio;
 
 extern int debug_net;
 
 /*
- * getopt flags (see apcopt.c)
+ * getopt flags (see apcoptd.c)
  */
 extern int show_version;
 extern char *cfgfile;
@@ -101,6 +82,14 @@ extern int rename_ups;
 extern int terminate_on_powerfail;
 extern int kill_ups_power;
 extern int dumb_mode_test;
+extern int go_background;
+
+
+/*
+ * In apceeprom.c
+ */
+extern int do_eeprom_programming(UPSINFO *ups);
+
 
 /*
  * In apcopt.c
@@ -110,34 +99,31 @@ extern int parse_options(int argc, char *argv[]);
 /*
  * In apcupsd.c
  */
-extern void error_out(char *file, int line, char *fmt,...);
-extern void error_exit(char *fmt,...);
 extern void powerfail (int ok);
 extern void logonfail (int ok);
 
 extern void thread_terminate (int sig);
 extern void clean_threads(void);
-extern void terminate (int sig);
+extern void apcupsd_terminate (int sig);
 extern void clear_files (void);
 extern void make_pid (void);
 
 /*
- * In apcserial.c
+ * In apcdevice.c
  */
-extern void setup_serial (UPSINFO *ups);
-extern void kill_power   (UPSINFO *ups);
-extern void save_dumb_status (UPSINFO *ups);
-extern int check_serial  (UPSINFO *ups);
-extern void prep_serial  (UPSINFO *ups);
-
-extern void timer_serial (int sig);
-extern void do_serial(UPSINFO *ups);
+void setup_device(UPSINFO *ups);
+extern void setup_serial(UPSINFO *ups);
+extern void kill_power(UPSINFO *ups);
+extern void save_dumb_status(UPSINFO *ups);
+extern int check_serial(UPSINFO *ups);
+extern void prep_device(UPSINFO *ups);
+extern void do_device(UPSINFO *ups);
 
 /*
  * In apcaction.c
  */
 extern void timer_action(int sig);
-extern void do_action(UPSINFO *ups, int ups_connected);
+extern void do_action(UPSINFO *ups);
 extern void generate_event(UPSINFO *ups, int event);
 
 /*
@@ -155,8 +141,6 @@ extern void log_struct (struct netdata *logstruct);
 
 extern int reconnect_master (UPSINFO *ups, int who);
 extern int prepare_master (UPSINFO *ups);
-extern void send_to_slaves (UPSINFO *ups);
-extern int send_to_each (UPSINFO *ups, int who);
 
 extern int reconnect_slave (UPSINFO *ups);
 extern int prepare_slave (UPSINFO *ups);
@@ -213,9 +197,10 @@ extern void setup_ups_alarm (UPSINFO *ups);
 extern void setup_ups_lowbatt_delay (UPSINFO *ups);
 extern void setup_ups_selftest (UPSINFO *ups);
 
-extern void get_UPS_capabilities (UPSINFO *ups);
-extern void read_extended (UPSINFO *ups);
-extern void setup_extended (UPSINFO *ups);
+extern void get_apc_model(UPSINFO *ups);
+extern void get_apc_capabilities(UPSINFO *ups);
+extern void read_extended (UPSINFO *ups); 
+extern void setup_extended(UPSINFO *ups); 
 
 /*
  * In apcnis.c
@@ -237,7 +222,7 @@ extern void stat_print(UPSINFO *ups, char *fmt, ...);
 /*
  * In apcevents.c
  */
-extern int truncate_events_file(UPSINFO *ups);
+extern int trim_eventfile(UPSINFO *ups);
 extern int output_events(int sockfd, FILE *events_file);
 
 
@@ -253,6 +238,7 @@ extern void do_reports (UPSINFO *ups);
 /*
  * In apcsmart.c
  */
+extern int apc_enable(UPSINFO *ups);
 extern int apc_write(char cmd, UPSINFO *ups);
 extern char *apc_read(UPSINFO *ups);
 extern char *apc_chat(char cmd, UPSINFO *ups);
@@ -260,9 +246,6 @@ extern int getline (char *s, int len, UPSINFO *ups);
 extern void UPSlinkCheck (UPSINFO *ups);
 extern char *smart_poll (char cmd, UPSINFO *ups);
 extern int fillUPS (UPSINFO *ups);
-extern void ReadUPS (UPSINFO *ups);
-extern void test_status_bits (UPSINFO *ups);
-extern void read_static_ups_data (UPSINFO *ups);
 
 /*
  * In apcsignal.c
@@ -276,21 +259,21 @@ extern void sleep_forever(void);
 /*
  * In apcipc.c
  */
-extern int init_ipc(UPSINFO *ups);
-extern int attach_ipc(UPSINFO *ups, int shmperm);
-extern int detach_ipc(UPSINFO *ups);
-extern int destroy_ipc(UPSINFO *ups);
+extern UPSINFO *new_ups(void);
+extern UPSINFO *attach_ups(UPSINFO *ups, int shmperm);
+extern int detach_ups(UPSINFO *ups);
+extern int destroy_ups(UPSINFO *ups);
 
-extern int create_semaphore(UPSINFO *ups);
-extern int attach_semaphore(UPSINFO *ups);
-extern int destroy_semaphore(UPSINFO *ups);
+#define read_lock(ups) _read_lock(__FILE__, __LINE__, (ups))
+#define read_unlock(ups) _read_unlock(__FILE__, __LINE__, (ups))
+#define write_lock(ups) _write_lock(__FILE__, __LINE__, (ups))
+#define write_unlock(ups) _write_unlock(__FILE__, __LINE__, (ups))
+#define read_lock(ups) _read_lock(__FILE__, __LINE__, (ups))
 
-extern int read_shmarea(UPSINFO *ups, int lock);
-extern int write_shmarea(UPSINFO *ups);
-extern int read_andlock_shmarea(UPSINFO *ups);
-extern int write_andunlock_shmarea(UPSINFO *ups);
-extern UPSINFO *lock_shmups(UPSINFO *ups);
-extern void unlock_shmups(UPSINFO *ups);
+extern int _read_lock(char *file, int line, UPSINFO *ups);
+extern int _read_unlock(char *file, int line, UPSINFO *ups);
+extern int _write_lock(char *file, int line, UPSINFO *ups);
+extern int _write_unlock(char *file, int line, UPSINFO *ups);
 
 /*
  * In apcexec.c
@@ -331,7 +314,26 @@ extern UPSINFO *getUpsByname(char *name);
 /*
  * In apcerror.c
  */
-extern void error_out(char *file, int line, char *fmt, ...);
-extern void error_exit(char *fmt, ...);
+extern void generic_error_out(char *file, int line, char *fmt, ...);
+extern void generic_error_exit(char *fmt, ...);
+
+/*  
+ * In asys.c 
+ */
+int amicrosleep(time_t sec, long usec);
+int avsnprintf(char *str, size_t size, const char *format, va_list ap);
+int asnprintf(char *str, size_t size, const char *fmt,  ...);
+char *astrncpy(char *dest, const char *src, int maxlen);
+char *astrncat(char *dest, const char *src, int maxlen);
+struct tm *localtime_r(const time_t *timep, struct tm *tm);
+char *afgets(char *s, int size, FILE *fd);;
+void *amalloc(size_t size);
+void *arealloc (void *buf, size_t size);
+void *acalloc (size_t size1, size_t size2);
+
+/* 
+ * In apcwinipc.c
+ */
+int winioctl(int fd, int func, int *addr);
 
 #endif /* _APC_EXTERN_H */
