@@ -70,6 +70,7 @@ typedef enum {
     APC_940_0020B,  /* APC cable number 940-0020B */
     APC_940_0020C,  /* APC cable number 940-0020C identical to 20B */
     APC_940_0023A,  /* APC cable number 940-0023A */
+    MAM_CABLE,      /* MAM cable for Alfatronic SPS500X */
     CUSTOM_SMART,   /* SMART cable smart          */
     APC_940_0024B,  /* APC cable number 940-0024B */
     APC_940_0024C,  /* APC cable number 940-0024C */
@@ -79,12 +80,14 @@ typedef enum {
     APC_940_0095B,  /* APC cable number 940-0095B */
     APC_940_0095C,  /* APC cable number 940-0095C */
     APC_NET,        /* Ethernet Link              */
+    USB_CABLE,      /* USB cable */
     APC_940_00XXX   /* APC cable number UNKNOWN   */
 } UpsCable;
 
 /* The order of these UpsModes is important!! */
 typedef enum {
     NO_UPS=0,   /* Default Disable      */
+    DUMB_UPS,   /* Dumb UPS driver      */
     BK,         /* Simple Signal        */
     SHAREBASIC, /* Simple Signal, Share */
     NETUPS,     /*                      */
@@ -94,6 +97,11 @@ typedef enum {
     SMART,      /* Smart Signal         */
     MATRIX,     /* Smart Signal         */
     SHARESMART, /* Smart Signal, Share  */
+    APCSMART_UPS, /* APC Smart UPS (any)*/
+    USB_UPS,    /* USB UPS driver       */
+    SNMP_UPS,   /* SNMP UPS driver      */
+    NETWORK_UPS,/* NETWORK UPS driver   */
+    TEST_UPS,   /* TEST UPS Driver      */
 } UpsMode;
 
 typedef enum {
@@ -176,13 +184,18 @@ typedef struct internalgeninfo {
  * master.
  */
 typedef struct slaveinfo {
-    int remote_state;
-    int socket;
-    int error;                    /* set when error message printed */
-    int errorcnt;                 /* count of errors */
+    int remote_state;                 /* state of master */
+    int disconnecting_slave;          /* set if old style slave */
+    int ms_errno;                     /* errno last error */
+    int socket;                       /* current open socket this slave */
+    int port;                         /* port */
+    int error;                        /* set when error message printed */
+    int errorcnt;                     /* count of errors */
+    time_t down_time;                 /* time slave was set RMT_DOWN */
     struct sockaddr_in addr;
-    char usermagic[APC_MAGIC_SIZE];
-    char name[MAXSTRING];
+    char usermagic[APC_MAGIC_SIZE];   /* Old style password */
+    char name[MAXTOKENLEN];           /* master/slave domain name (or IP) */
+    char password[MAXTOKENLEN];       /* for CRAM-MD5 authentication */
 } SLAVEINFO;
 
 /* 
@@ -218,7 +231,7 @@ typedef struct netdata {
  * risk to have users reading the shared memory with an old
  * nonvalid upsinfo structure. 
  */
-#define UPSINFO_VERSION 10
+#define UPSINFO_VERSION 12
 /*
  * There is no need to change the following, but you can if
  * you want, but it must be at least 4 characters to match
@@ -233,30 +246,25 @@ typedef struct upsinfo {
     char release[20];
 
     int fd;                       /* serial port file descriptor */
-    int sp_flags;                 /* serial port flags on DUMB UPSes */
 
     /* UPS capability array and codes */
-    char UPS_Cap[CI_MAXCI+1];
-    char UPS_Cmd[CI_MAXCI+1];
+    char     UPS_Cap[CI_MAXCI+1];     /* TRUE if UPS has capability */
+    unsigned int UPS_Cmd[CI_MAXCI+1]; /* Command or function code */
+
+    char *buf;                        /* scratch buffer */
+    int buf_len;                      /* buffer length */
 
 
     INTERNALGENINFO cable;
     INTERNALGENINFO enable_access;
     INTERNALGENINFO nologin;
     INTERNALGENINFO mode;
-    INTERNALGENINFO class;
+    INTERNALGENINFO upsclass;
     INTERNALGENINFO sharenet;
 
     int num_execed_children;      /* children created in execute_command() */
 
     /* Internal state flags set in response to UPS condition */
-    int OnBatt;                   /* set when UPS switches to battery power */
-    int BattLow;                  /* set if battery charge is low */
-    int LineDown;
-    int ChangeBatt;               /* set when UPS says battery should be changed */
-    int RestartDaemon;            /* set if more than 100 errors on serial port */
-    int BatteryUp;                /* set when Running on UPS message sent */
-    int CommLost;                 /* Connection to UPS lost */
     time_t ShutDown;              /* set when doing shutdown */
     time_t SelfTest;              /* start time of self test */
     time_t LastSelfTest;          /* time of last self test */
@@ -271,10 +279,9 @@ typedef struct upsinfo {
     time_t last_master_connect_time; /* last time master connected */
     int num_xfers;                /* number of times on batteries */
     int cum_time_on_batt;         /* total time on batteries since startup */
-    int OldOnBatt;
-    int OldBattLow;
-    int FastPoll;                 /* Poll UPS faster during power failure */
-
+    int wait_time;                /* suggested wait time for drivers in 
+                                   * device_check_state() 
+                                   */
 
     /* Items reported by smart UPS */
     /* Volatile items -- i.e. they change with the state of the UPS */
@@ -283,7 +290,9 @@ typedef struct upsinfo {
     unsigned int reg2;            /* register 2 */
     unsigned int reg3;            /* register 3 */
     unsigned int dipsw;           /* dip switch info */
-    char G[8];                    /* reason for last switch to batteries */
+    unsigned int InputPhase;      /* The current AC input phase. */
+    unsigned int OutputPhase;     /* The current AC output phase. */
+    char G[22];                   /* reason for last switch to batteries */
     char X[8];                    /* results of last seft test */
     double BattChg;               /* remaining UPS charge % */
     double LineMin;               /* min line voltage seen */
@@ -292,14 +301,15 @@ typedef struct upsinfo {
     double LineFreq;              /* line freq. */
     double LineVoltage;           /* Line Voltage */
     double OutputVoltage;         /* Output Voltage */
+    double OutputFreq;            /* Output Frequency */
+    double OutputCurrent;         /* Output Current */
     double UPSTemp;               /* UPS internal temperature */
     double BattVoltage;           /* Actual Battery voltage -- about 24V */
-    double LastSTTime;            /* hours since last self test -- not yet implemented */
-    unsigned int Status;          /* UPS status */
+    double LastSTTime; /* hours since last self test -- not yet implemented */
+    int32_t Status;               /* UPS status (Bitmapped) */
     double TimeLeft;              /* Est. time UPS can run on batt. */
     double humidity;              /* Humidity */
     double ambtemp;               /* Ambient temperature */
-    int LineLevel;                /* -1=LineVoltage low; 0=normal; 1=High */
     char eprom[500];              /* Eprom values */
 
     /* Items reported by smart UPS */
@@ -317,49 +327,49 @@ typedef struct upsinfo {
     char birth[20];               /* manufacture date */
     char serial[32];              /* serial number */
     char battdat[20];             /* battery installation date */
-    char selftest[8];             /* selftest interval in ASCII */
+    char selftest[9];             /* selftest interval as ASCII */
     char firmrev[20];             /* firmware revision */
+    char upsname[UPSNAMELEN];     /* UPS internal name */
     char upsmodel[20];            /* ups model number */
     char sensitivity[8];          /* sensitivity to line fluxuations */
     char beepstate[8];            /* when to beep on power failure. */
+    char selftestmsg[80];
 
     /* Items specified from config file */
+    int PoweredByUPS;             /* The only bit left out from the bitmap */
     int annoy;
     int maxtime;
-    int annoydelay;                   /* delay before annoying users with logoff request */
-    int killdelay;                    /* delay after pwrfail before issuing UPS shutdown */
+    int annoydelay;      /* delay before annoying users with logoff request */
+    int onbattdelay;     /* delay before reacting to a power failure */
+    int killdelay;       /* delay after pwrfail before issuing UPS shutdown */
     int nologin_time;
     int nologin_file;
     int stattime;
     int datatime;
     int sysfac;
     int reports;
-    int nettime;                     /* Time interval for master to send to slaves */
+    int nettime;         /* Time interval for master to send to slaves */
     int percent;                     /* shutdown when batt % less than this */
     int runtime;                     /* shutdown when runtime less than this */
+    char nisip[64];                  /* IP for NIS */
     int statusport;                  /* TCP port for STATUS */
     int netstats;                    /* turn on/off network status */
     int logstats;                    /* turn on/off logging of status info */
-    char device[MAXSTRING];          /**/
-    char configfile[FILENAME_MAX];   /* config filename */
-    char statfile[FILENAME_MAX];     /* status filename */
-    char eventfile[FILENAME_MAX];    /* temp events file */
+    char device[MAXSTRING];          /* device name in use */
+    char configfile[APC_FILENAME_MAX];   /* config filename */
+    char statfile[APC_FILENAME_MAX];     /* status filename */
+    char eventfile[APC_FILENAME_MAX];    /* temp events file */
+    int eventfilemax;                /* max size of eventfile in kilobytes */
     int event_fd;                    /* fd for eventfile */
 
     int NetUpsPort;                  /* Our communication port */
-    char master_name[FILENAME_MAX];  /**/
+    char master_name[APC_FILENAME_MAX];  /**/
 
-    char lockpath[FILENAME_MAX];     /* BSC, made static -RF */
+    char lockpath[APC_FILENAME_MAX];     /* BSC, made static -RF */
     int lockfile;                    /* BSC */
 
     char usermagic[APC_MAGIC_SIZE];  /* security id string */
-
-    /* reasons for shutdown */
-    int load;                        /* set when BatLoad <= percent */
-    int timedout;                    /* set when time on batts > maxtime */
-    int timelout;                    /* set when TimeLeft <= runtime */
-    int emergencydown;               /* battery power has failed */
-    int remotedown;                  /* remote shutdown */
+    int ChangeBattCounter;           /* For UPS_REPLACEBATT, see apcaction.c */
 
     int remote_state;                /**/
 
@@ -368,9 +378,6 @@ typedef struct upsinfo {
      * possible to be backward compatible. All moved variables are kept as
      * reserved and all the new variables are at the end of the structure.
      */
-    char upsname[UPSNAMELEN];   /* UPS config name */
-    int PoweredByUPS;           /* Flag: is this UPS local or remote ?  */
-    int has_done_setup;         /* setup_serial already done */
 
 #ifndef HAVE_PTHREADS
     /*
@@ -380,21 +387,13 @@ typedef struct upsinfo {
      * Don't use shmUPS directly: call ipc functions to get it or
      * unpredictable results may happen.
      */
-    struct upsinfo *shmUPS;     /* Pointer to shared IPC UPSINFO structure */
     int sem_id;                 /* Semaphore ID */
     int shm_id;                 /* Shared memory ID */
     struct sembuf semUPS[NUM_SEM_OPER]; /* Semaphore operators */
     int idshmUPS;               /* key of shared memory area */
     int idsemUPS;               /* key of semphores */
-    int shm_OK;                 /* shm area updated */
 #endif
 
-    /*
-     * Saved termio structures.
-     */
-    struct termios oldtio;
-    struct termios newtio;
-
     /*
      * Linked list of UPSes used in apclist.c
      */
@@ -402,7 +401,11 @@ typedef struct upsinfo {
     struct upsinfo *next;
 #ifdef HAVE_PTHREADS
     pthread_mutex_t mutex;
+    int refcnt;                       /* thread attach count */
 #endif
+
+    struct upsdriver *driver;       /* UPS driver for this UPSINFO */
+    void *driver_internal_data;     /* Driver private data */
 } UPSINFO;
 
 /* Used only in apcaccess.c */
